@@ -36,12 +36,12 @@ main()
     /* Open file and create group. */
     if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                             H5P_DEFAULT)) < 0) ERR;
-    if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+    if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
     /* Write an array of bools. */
     dims[0] = DIM1_LEN;
     if ((spaceid = H5Screate_simple(1, dims, dims)) < 0) ERR;
-    if ((datasetid = H5Dcreate(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
+    if ((datasetid = H5Dcreate1(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
                                spaceid, H5P_DEFAULT)) < 0) ERR;
     if (H5Dwrite(datasetid, H5T_NATIVE_HBOOL, H5S_ALL, H5S_ALL, H5P_DEFAULT,
                  bool_out) < 0) ERR;
@@ -81,7 +81,7 @@ main()
         /* Create file and group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP2_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP2_NAME, 0)) < 0) ERR;
 
         /* Create a space corresponding to these three dimensions. */
         dims[0] = 0;
@@ -100,9 +100,9 @@ main()
         if (H5Pset_chunk(cparmsid, NDIMS, dims) < 0) ERR;
 
         /* Create two variables which use this space. */
-        if ((pres_dsid = H5Dcreate(grpid, PRES_NAME, H5T_NATIVE_FLOAT,
+        if ((pres_dsid = H5Dcreate1(grpid, PRES_NAME, H5T_NATIVE_FLOAT,
                                    spaceid, cparmsid)) < 0) ERR;
-        if ((temp_dsid = H5Dcreate(grpid, TEMP_NAME, H5T_NATIVE_FLOAT,
+        if ((temp_dsid = H5Dcreate1(grpid, TEMP_NAME, H5T_NATIVE_FLOAT,
                                    spaceid, cparmsid)) < 0) ERR;
 
         /* Get the spaceid and check various things. */
@@ -198,7 +198,7 @@ main()
         /* Open file and create group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
         /* Write an array of bools, with compression. */
         dims[0] = DIM1_LEN;
@@ -207,7 +207,7 @@ main()
         if (H5Pset_chunk(propid, 1, dims)) ERR;
         if (H5Pset_deflate(propid, DEFLATE_LEVEL)) ERR;
         if ((spaceid = H5Screate_simple(1, dims, dims)) < 0) ERR;
-        if ((datasetid = H5Dcreate(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
+        if ((datasetid = H5Dcreate1(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
                                    spaceid, propid)) < 0) ERR;
         if (H5Dwrite(datasetid, H5T_NATIVE_HBOOL, H5S_ALL, H5S_ALL, H5P_DEFAULT,
                      bool_out) < 0) ERR;
@@ -220,7 +220,7 @@ main()
 
         /* Now reopen the file and check. */
         if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+        if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
         if ((datasetid = H5Dopen1(grpid, VAR_BOOL_NAME)) < 0) ERR;
         if ((propid = H5Dget_create_plist(datasetid)) < 0) ERR;
 
@@ -256,7 +256,7 @@ main()
         /* Open file and create group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
         /* Write an array of bools, with compression, fletcher32
          * checksum, shuffle filters. Like a hoogie with "the works." */
@@ -268,7 +268,7 @@ main()
         if (H5Pset_deflate(propid, DEFLATE_LEVEL)) ERR;
         if (H5Pset_fletcher32(propid)) ERR;
         if ((spaceid = H5Screate_simple(1, dims, dims)) < 0) ERR;
-        if ((datasetid = H5Dcreate(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
+        if ((datasetid = H5Dcreate1(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
                                    spaceid, propid)) < 0) ERR;
         if (H5Dwrite(datasetid, H5T_NATIVE_HBOOL, H5S_ALL, H5S_ALL, H5P_DEFAULT,
                      bool_out) < 0) ERR;
@@ -281,7 +281,7 @@ main()
 
         /* Now reopen the file and check. */
         if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+        if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
         if ((datasetid = H5Dopen1(grpid, VAR_BOOL_NAME)) < 0) ERR;
         if ((propid = H5Dget_create_plist(datasetid)) < 0) ERR;
 
@@ -339,16 +339,16 @@ main()
         /* Open file and create group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
         /* Create a dataset of native endian. */
         dims[0] = DIM1_LEN;
         if ((spaceid = H5Screate_simple(1, dims, dims)) < 0) ERR;
-        if ((native_did = H5Dcreate(grpid, NATIVE_VAR_NAME, H5T_NATIVE_INT,
+        if ((native_did = H5Dcreate1(grpid, NATIVE_VAR_NAME, H5T_NATIVE_INT,
                                     spaceid, H5P_DEFAULT)) < 0) ERR;
-        if ((le_did = H5Dcreate(grpid, LE_VAR_NAME, H5T_STD_I32LE,
+        if ((le_did = H5Dcreate1(grpid, LE_VAR_NAME, H5T_STD_I32LE,
                                 spaceid, H5P_DEFAULT)) < 0) ERR;
-        if ((be_did = H5Dcreate(grpid, BE_VAR_NAME, H5T_STD_I32BE,
+        if ((be_did = H5Dcreate1(grpid, BE_VAR_NAME, H5T_STD_I32BE,
                                 spaceid, H5P_DEFAULT)) < 0) ERR;
         if (H5Dwrite(native_did, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,
                      data) < 0) ERR;
@@ -366,7 +366,7 @@ main()
 
         /* Now reopen the file and check. */
         if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+        if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
 
         if ((native_did = H5Dopen1(grpid, NATIVE_VAR_NAME)) < 0) ERR;
         if ((typeid = H5Dget_type(native_did)) < 0) ERR;
@@ -409,7 +409,7 @@ main()
     }
 
     SUMMARIZE_ERR;
-#ifdef USE_SZIP
+#ifdef HAVE_H5Z_SZIP
     printf("*** Checking szip functionality...");
 #define SZIP_VAR_NAME "szip_var"
 #define SZIP_DIM1_LEN 32
@@ -432,7 +432,7 @@ main()
         /* Open file and create group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
         /* Create dataset creation property list. */
         if ((plistid = H5Pcreate(H5P_DATASET_CREATE)) < 0) ERR;
@@ -474,7 +474,7 @@ main()
             int i;
 
             if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-            if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+            if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
 
             if ((native_did = H5Dopen1(grpid, SZIP_VAR_NAME)) < 0) ERR;
 
@@ -520,7 +520,7 @@ main()
             sprintf(file_name, "%s_%s.h5", TEST_NAME, desc[f]);
             if ((fileid = H5Fcreate(file_name, H5F_ACC_TRUNC, H5P_DEFAULT,
                                     H5P_DEFAULT)) < 0) ERR;
-            if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+            if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
             /* Create dataset creation property list. */
             if ((plistid = H5Pcreate(H5P_DATASET_CREATE)) < 0) ERR;
@@ -579,7 +579,7 @@ main()
                 int i;
 
                 if ((fileid = H5Fopen(file_name, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-                if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+                if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
 
                 if ((native_did = H5Dopen1(grpid, BOTH_VAR_NAME)) < 0) ERR;
 
@@ -596,6 +596,6 @@ main()
         } /* next file */
     }
     SUMMARIZE_ERR;
-#endif /* USE_SZIP */
+#endif /* HAVE_H5Z_SZIP */
     FINAL_RESULTS;
 }
