@@ -12,6 +12,11 @@
 #include "zincludes.h"
 #include <math.h> /* For pow() used below. */
 
+/* Mnemonics */
+#define CREATE 0
+#define NOCREATE 1
+
+
 #ifdef LOGGING
 static void
 reportchunking(const char* title, NC_VAR_INFO_T* var)
@@ -107,10 +112,7 @@ ncz_find_default_chunksizes2(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var)
     double total_chunk_size;
 #endif
 
-    if (var->type_info->nc_type_class == NC_STRING)
-	type_size = sizeof(char *);
-    else
-	type_size = var->type_info->size;
+    type_size = var->type_info->size;
 
 #ifdef LOGGING
     /* Later this will become the total number of bytes in the default
@@ -381,7 +383,7 @@ NCZ_def_var(int ncid, const char *name, nc_type xtype, int ndims,
     zvar->common.file = h5;
     zvar->scalar = (ndims == 0 ? 1 : 0);
 
-    zvar->dimension_separator = ncrc_getglobalstate()->zarr.dimension_separator;
+    zvar->dimension_separator = NC_getglobalstate()->zarr.dimension_separator;
     assert(zvar->dimension_separator != 0);
 
     /* Set these state flags for the var. */
@@ -389,8 +391,11 @@ NCZ_def_var(int ncid, const char *name, nc_type xtype, int ndims,
     var->meta_read = NC_TRUE;
     var->atts_read = NC_TRUE;
 
+#ifdef ENABLE_NCZARR_FILTERS
     /* Set the filter list */
+    assert(var->filters == NULL);
     var->filters = (void*)nclistnew();
+#endif
 
     /* Point to the type, and increment its ref. count */
     var->type_info = type;
@@ -405,7 +410,7 @@ var->type_info->rc++;
     /* Set variables no_fill to match the database default unless the
      * variable type is variable length (NC_STRING or NC_VLEN) or is
      * user-defined type. */
-    if (var->type_info->nc_type_class < NC_STRING)
+    if (var->type_info->nc_type_class <= NC_STRING)
 	var->no_fill = (h5->fill_mode == NC_NOFILL);
 
     /* Assign dimensions to the variable. At the same time, check to
@@ -435,36 +440,35 @@ var->type_info->rc++;
      * variables which may be contiguous.) */
     LOG((4, "allocating array of %d size_t to hold chunksizes for var %s",
 	 var->ndims, var->hdr.name));
-    if (var->ndims) {
-	if (!(var->chunksizes = calloc(var->ndims, sizeof(size_t))))
-	    BAIL(NC_ENOMEM);
-	if ((retval = ncz_find_default_chunksizes2(grp, var)))
-	    BAIL(retval);
-    } else {
-	/* Pretend that scalars are like var[1] */
-	if (!(var->chunksizes = calloc(1, sizeof(size_t))))
-	    BAIL(NC_ENOMEM);
-	var->chunksizes[0] = 1;
+    if(!var->chunksizes) {
+	if(var->ndims) {
+            if (!(var->chunksizes = calloc(var->ndims, sizeof(size_t))))
+	        BAIL(NC_ENOMEM);
+	    if ((retval = ncz_find_default_chunksizes2(grp, var)))
+	        BAIL(retval);
+        } else {
+	    /* Pretend that scalars are like var[1] */
+	    if (!(var->chunksizes = calloc(1, sizeof(size_t))))
+	        BAIL(NC_ENOMEM);
+	    var->chunksizes[0] = 1;
+	}
     }
     
     /* Compute the chunksize cross product */
     zvar->chunkproduct = 1;
-    for(d=0;d<var->ndims+zvar->scalar;d++) {zvar->chunkproduct *= var->chunksizes[d];}
+    if(!zvar->scalar)
+        {for(d=0;d<var->ndims;d++) {zvar->chunkproduct *= var->chunksizes[d];}}
     zvar->chunksize = zvar->chunkproduct * var->type_info->size;
 
     /* Override the cache setting to use NCZarr defaults */
-    var->chunk_cache_size = CHUNK_CACHE_SIZE_NCZARR;
-    var->chunk_cache_nelems = ceildiv(var->chunk_cache_size,zvar->chunksize);
-    var->chunk_cache_preemption = 1; /* not used */
+    var->chunkcache.size = CHUNK_CACHE_SIZE_NCZARR;
+    var->chunkcache.nelems = ceildiv(var->chunkcache.size,zvar->chunksize);
+    var->chunkcache.preemption = 1; /* not used */
 
     /* Create the cache */
     if((retval=NCZ_create_chunk_cache(var,zvar->chunkproduct*var->type_info->size,zvar->dimension_separator,&zvar->cache)))
 	BAIL(retval);
 
-    /* Is this a variable with a chunksize greater than the current cache size? */
-    if ((retval = NCZ_adjust_var_cache(grp, var)))
-	BAIL(retval);
-
     /* Return the varid. */
     if (varidp)
 	*varidp = var->hdr.id;
@@ -474,7 +478,6 @@ exit:
     if (type)
 	if ((retval = nc4_type_free(type)))
 	    BAILLOG(retval);
-
     return ZUNTRACE(retval);
 }
 
@@ -513,7 +516,8 @@ static int
 ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 		 int *unused2, int *fletcher32, int *storagep,
 		 const size_t *chunksizes, int *no_fill,
-		 const void *fill_value, int *endianness)
+		 const void *fill_value, int *endianness,
+		 int *quantize_mode, int *nsd)
 {
     NC_GRP_INFO_T *grp;
     NC_FILE_INFO_T *h5;
@@ -525,13 +529,16 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 
     LOG((2, "%s: ncid 0x%x varid %d", __func__, ncid, varid));
 
-    ZTRACE(2,"ncid=%d varid=%d shuffle=%d fletcher32=%d no_fill=%d, fill_value=%p endianness=%d",
+    ZTRACE(2,"ncid=%d varid=%d shuffle=%d fletcher32=%d no_fill=%d, fill_value=%p endianness=%d quantize_mode=%d nsd=%d",
            ncid,varid,
 	   (shuffle?*shuffle:-1),
    	   (fletcher32?*fletcher32:-1),
 	   (no_fill?*no_fill:-1),
 	   fill_value,
-	   (endianness?*endianness:-1));
+	   (endianness?*endianness:-1),
+	   (quantize_mode?*quantize_mode:-1),
+   	   (nsd?*nsd:-1)
+	   );
 
     /* Find info for this file and group, and set pointer to each. */
     if ((retval = nc4_find_nc_grp_h5(ncid, NULL, &grp, &h5)))
@@ -547,14 +554,18 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 	{retval = NC_ENOTVAR; goto done;}
     assert(var && var->hdr.id == varid);
 
+    zvar = var->format_var_info;
+
     ZTRACEMORE(1,"\tstoragep=%d chunksizes=%s",(storagep?*storagep:-1),(chunksizes?nczprint_sizevector(var->ndims,chunksizes):"null"));
     
     /* Can't turn on parallel and deflate/fletcher32/szip/shuffle
      * before HDF5 1.10.3. */
+#ifdef ENABLE_NCZARR_FILTERS
 #ifndef HDF5_SUPPORTS_PAR_FILTERS
     if (h5->parallel == NC_TRUE)
 	if (nclistlength(((NClist*)var->filters)) > 0  || fletcher32 || shuffle)
 	    {retval = NC_EINVAL; goto done;}
+#endif
 #endif
 
     /* If the HDF5 dataset has already been created, then it is too
@@ -595,17 +606,21 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 #endif
 
     /* Shuffle filter? */
-    if (shuffle)
-    {
-	if(*shuffle) var->shuffle = *shuffle;
-	var->storage = NC_CHUNKED;
+    if (shuffle && *shuffle) {
+	    retval = nc_inq_var_filter_info(ncid,varid,H5Z_FILTER_SHUFFLE,NULL,NULL);
+	    if(!retval || retval == NC_ENOFILTER) {
+	        if((retval = NCZ_def_var_filter(ncid,varid,H5Z_FILTER_SHUFFLE,0,NULL))) return retval;
+                var->storage = NC_CHUNKED;
+	    }
     }
 
     /* Fletcher32 checksum error protection? */
-    if (fletcher32)
-    {
-	if(*fletcher32) var->fletcher32 = *fletcher32;
-	var->storage = NC_CHUNKED;
+    if (fletcher32 && fletcher32) {
+	retval = nc_inq_var_filter_info(ncid,varid,H5Z_FILTER_FLETCHER32,NULL,NULL);
+	if(!retval || retval == NC_ENOFILTER) {
+	    if((retval = NCZ_def_var_filter(ncid,varid,H5Z_FILTER_FLETCHER32,0,NULL))) return retval;
+            var->storage = NC_CHUNKED;
+	    }
     }
 
     /* Handle storage settings. */
@@ -617,8 +632,10 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 	 * no filters in use for this data. */
 	if (storage != NC_CHUNKED)
 	{
-	    if (nclistlength(((NClist*)var->filters)) > 0 || var->fletcher32 || var->shuffle)
+#ifdef NCZARR_FILTERS
+	    if (nclistlength(((NClist*)var->filters)) > 0)
 		{retval = NC_EINVAL; goto done;}
+#endif
 	    for (d = 0; d < var->ndims; d++)
 		if (var->dim[d]->unlimited)
 		    {retval = NC_EINVAL; goto done;}
@@ -630,7 +647,7 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 	    {retval = NC_EINVAL; goto done;}
 	} else if (storage == NC_CHUNKED && var->ndims > 0) {
 	    var->storage = NC_CHUNKED;
-
+	    
 	    /* If the user provided chunksizes, check that they are valid
 	     * and that their total size of chunk is less than 4 GB. */
 	    if (chunksizes)
@@ -658,6 +675,7 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 	    int anyzero = 0; /* check for any zero length chunksizes */
 	    zvar = var->format_var_info;
 	    assert(zvar->cache != NULL);
+	    zvar->cache->valid = 0;
 	    if(chunksizes) {
 		for (d = 0; d < var->ndims; d++) {
 		    var->chunksizes[d] = chunksizes[d];
@@ -675,10 +693,9 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 	    for (d = 0; d < var->ndims; d++)
 		zvar->chunkproduct *= var->chunksizes[d];
             zvar->chunksize = zvar->chunkproduct * var->type_info->size;
-	    /* Adjust the cache. */
-	    if ((retval = NCZ_adjust_var_cache(grp, var)))
-		goto done;
 	}
+	/* Adjust cache */
+        if((retval = NCZ_adjust_var_cache(var))) goto done;
     
 #ifdef LOGGING
 	{
@@ -707,7 +724,7 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
     }
 
     /* Are we setting a fill value? */
-    if (fill_value && !var->no_fill)
+    if (fill_value && no_fill && !(*no_fill))
     {
 	/* Copy the fill_value. */
 	LOG((4, "Copying fill value into metadata for variable %s",
@@ -718,10 +735,17 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 	if (retval && retval != NC_ENOTATT)
 	    goto done;
 
-	/* Create a _FillValue attribute. */
+        /* Create a _FillValue attribute; will also fill in var->fill_value */
 	if ((retval = nc_put_att(ncid, varid, _FillValue, var->type_info->hdr.id,
 				 1, fill_value)))
 	    goto done;
+        /* Reclaim any existing fill_chunk */
+        if((retval = NCZ_reclaim_fill_chunk(zvar->cache))) goto done;
+    } else if (var->fill_value && no_fill && (*no_fill)) { /* Turning off fill value? */
+        /* If there's a _FillValue attribute, delete it. */
+        retval = NCZ_del_att(ncid, varid, _FillValue);
+        if (retval && retval != NC_ENOTATT) return retval;
+	if((retval = NCZ_reclaim_fill_value(var))) return retval;
     }
 
     /* Is the user setting the endianness? */
@@ -750,6 +774,66 @@ ncz_def_var_extra(int ncid, int varid, int *shuffle, int *unused1,
 	var->endianness = *endianness;
     }
 
+    /* Remember quantization settings. They will be used when data are
+     * written.
+     * Code block is identical to one in hdf5var.c---consider functionalizing */
+    if (quantize_mode)
+    {
+	/* Only four valid mode settings. */
+	if (*quantize_mode != NC_NOQUANTIZE &&
+	    *quantize_mode != NC_QUANTIZE_BITGROOM &&
+	    *quantize_mode != NC_QUANTIZE_GRANULARBR &&
+	    *quantize_mode != NC_QUANTIZE_BITROUND)
+	    return NC_EINVAL;
+
+	if (*quantize_mode == NC_QUANTIZE_BITGROOM ||
+	    *quantize_mode == NC_QUANTIZE_GRANULARBR ||
+	    *quantize_mode == NC_QUANTIZE_BITROUND)
+	  {
+
+	    /* Only float and double types can have quantization. */
+	    if (var->type_info->hdr.id != NC_FLOAT &&
+		var->type_info->hdr.id != NC_DOUBLE)
+		return NC_EINVAL;
+	    
+	    /* All quantization codecs require number of significant digits */
+	    if (!nsd)
+		return NC_EINVAL;
+
+	    /* NSD must be in range. */
+	    if (*nsd <= 0)
+		return NC_EINVAL;
+
+	    if (*quantize_mode == NC_QUANTIZE_BITGROOM ||
+		*quantize_mode == NC_QUANTIZE_GRANULARBR)
+	      {
+		if (var->type_info->hdr.id == NC_FLOAT &&
+		    *nsd > NC_QUANTIZE_MAX_FLOAT_NSD)
+		  return NC_EINVAL;
+		if (var->type_info->hdr.id == NC_DOUBLE &&
+		    *nsd > NC_QUANTIZE_MAX_DOUBLE_NSD)
+		  return NC_EINVAL;
+	      }
+	    else if (*quantize_mode == NC_QUANTIZE_BITROUND)
+	      {
+		if (var->type_info->hdr.id == NC_FLOAT &&
+		    *nsd > NC_QUANTIZE_MAX_FLOAT_NSB)
+		  return NC_EINVAL;
+		if (var->type_info->hdr.id == NC_DOUBLE &&
+		    *nsd > NC_QUANTIZE_MAX_DOUBLE_NSB)
+		  return NC_EINVAL;
+	      }
+
+	    var->nsd = *nsd;
+	}
+	
+	var->quantize_mode = *quantize_mode;
+
+	/* If quantization is turned off, then set nsd to 0. */
+	if (*quantize_mode == NC_NOQUANTIZE)
+	    var->nsd = 0;
+    }
+
 done:
     return ZUNTRACE(retval);
 }
@@ -782,7 +866,7 @@ NCZ_def_var_deflate(int ncid, int varid, int shuffle, int deflate,
     int stat = NC_NOERR;
     unsigned int level = (unsigned int)deflate_level;
     /* Set shuffle first */
-    if((stat = ncz_def_var_extra(ncid, varid, &shuffle, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL))) goto done;
+    if((stat = ncz_def_var_extra(ncid, varid, &shuffle, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL))) goto done;
     if(deflate)
 	stat = nc_def_var_filter(ncid, varid, H5Z_FILTER_DEFLATE,1,&level);
     if(stat) goto done;
@@ -812,7 +896,7 @@ int
 NCZ_def_var_fletcher32(int ncid, int varid, int fletcher32)
 {
     return ncz_def_var_extra(ncid, varid, NULL, NULL, NULL, &fletcher32,
-			    NULL, NULL, NULL, NULL, NULL);
+			    NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 }
 
 /**
@@ -841,7 +925,7 @@ int
 NCZ_def_var_chunking(int ncid, int varid, int contiguous, const size_t *chunksizesp)
 {
     return ncz_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL,
-			    &contiguous, chunksizesp, NULL, NULL, NULL);
+			    &contiguous, chunksizesp, NULL, NULL, NULL, NULL, NULL);
 }
 
 /**
@@ -886,7 +970,7 @@ ncz_def_var_chunking_ints(int ncid, int varid, int contiguous, int *chunksizesp)
 	cs[i] = chunksizesp[i];
 
     retval = ncz_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL,
-			      &contiguous, cs, NULL, NULL, NULL);
+			      &contiguous, cs, NULL, NULL, NULL, NULL, NULL);
 
     if (var->ndims)
 	free(cs);
@@ -920,7 +1004,7 @@ int
 NCZ_def_var_fill(int ncid, int varid, int no_fill, const void *fill_value)
 {
     return ncz_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL, NULL,
-			    NULL, &no_fill, fill_value, NULL);
+			    NULL, &no_fill, fill_value, NULL, NULL, NULL);
 }
 
 /**
@@ -949,9 +1033,165 @@ int
 NCZ_def_var_endian(int ncid, int varid, int endianness)
 {
     return ncz_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL, NULL,
-			    NULL, NULL, NULL, &endianness);
+			    NULL, NULL, NULL, &endianness, NULL, NULL);
 }
 
+/**
+ * @internal Set quantization settings on a variable. This is
+ * called by nc_def_var_quantize().
+ *
+ * Quantization allows the user to specify a number of significant
+ * digits for variables of type ::NC_FLOAT or ::NC_DOUBLE. (Attempting
+ * to set quantize for other types will result in an ::NC_EINVAL
+ * error.)
+ *
+ * When quantize is turned on, and the number of significant digits
+ * (NSD) has been specified, then the netCDF library will quantize according
+ * to the selected algorithm. BitGroom interprets NSD as decimal digits
+ * will apply all zeros or all ones (alternating) to bits which are not 
+ * needed to specify the value to the number of significant decimal digits. 
+ * BitGroom retain the same number of bits for all values of a variable. 
+ * BitRound (BR) interprets NSD as binary digits (i.e., bits) and keeps the
+ * the user-specified number of significant bits then rounds the result
+ * to the nearest representable number according to IEEE rounding rules.
+ * BG and BR both retain a uniform number of significant bits for all 
+ * values of a variable. Granular BitRound interprest NSD as decimal
+ * digits. GranularBR determines the number of bits to necessary to 
+ * retain the user-specified number of significant digits individually
+ * for every value of the variable. GranularBR then applies the BR
+ * quantization algorithm on a granular, value-by-value, rather than
+ * uniformly for the entire variable. GranularBR quantizes more bits
+ * than BG, and is thus more compressive and less accurate than BG.
+ * BR knows bits and makes no guarantees about decimal precision.
+ * All quantization algorithms change the values of the data, and make 
+ * it more compressible.
+ *
+ * Quantizing the data does not reduce the size of the data on disk,
+ * but combining quantize with compression will allow for better
+ * compression. Since the data values are changed, the use of quantize
+ * and compression such as deflate constitute lossy compression.
+ *
+ * Producers of large datasets may find that using quantize with
+ * compression will result in significant improvent in the final data
+ * size.
+ *
+ * Variables which use quantize will have added an attribute with name
+ * ::NC_QUANTIZE_BITGROOM_ATT_NAME, ::NC_QUANTIZE_GRANULARBR_ATT_NAME, 
+ * or ::NC_QUANTIZE_BITROUND_ATT_NAME that contains the number of 
+ * significant digits. Users should not delete or change this attribute. 
+ * This is the only record that quantize has been applied to the data.
+ *
+ * As with the deflate settings, quantize settings may only be
+ * modified before the first call to nc_enddef(). Once nc_enddef() is
+ * called for the file, quantize settings for any variable in the file
+ * may not be changed.
+ *
+ * Use of quantization is fully backwards compatible with existing
+ * versions and packages that can read compressed netCDF data. A
+ * variable which has been quantized is readable to older versions of
+ * the netCDF libraries, and to netCDF-Java.
+ *
+ * @param ncid File ID.
+ * @param varid Variable ID. NC_GLOBAL may not be used.
+ * @param quantize_mode Quantization mode. May be ::NC_NOQUANTIZE or
+ * ::NC_QUANTIZE_BITGROOM, ::NC_QUANTIZE_BITROUND or ::NC_QUANTIZE_GRANULARBR.
+ * @param nsd Number of significant digits (either decimal or binary). 
+ * May be any integer from 1 to ::NC_QUANTIZE_MAX_FLOAT_NSD (for variables 
+ * of type ::NC_FLOAT) or ::NC_QUANTIZE_MAX_DOUBLE_NSD (for variables 
+ * of type ::NC_DOUBLE) for mode ::NC_QUANTIZE_BITGROOM and mode
+ * ::NC_QUANTIZE_GRANULARBR. May be any integer from 1 to 
+ * ::NC_QUANTIZE_MAX_FLOAT_NSB (for variables of type ::NC_FLOAT) or 
+ * ::NC_QUANTIZE_MAX_DOUBLE_NSB (for variables of type ::NC_DOUBLE) 
+ * for mode ::NC_QUANTIZE_BITROUND.
+ *
+ * @returns ::NC_NOERR No error.
+ * @returns ::NC_EBADID Bad ncid.
+ * @returns ::NC_ENOTVAR Invalid variable ID.
+ * @returns ::NC_ENOTNC4 Attempting netcdf-4 operation on file that is
+ * not netCDF-4/HDF5.
+ * @returns ::NC_ELATEDEF Too late to change settings for this variable.
+ * @returns ::NC_ENOTINDEFINE Not in define mode.
+ * @returns ::NC_EINVAL Invalid input
+ * @author Ed Hartnett, Dennis Heimbigner
+ */
+int
+NCZ_def_var_quantize(int ncid, int varid, int quantize_mode, int nsd)
+{
+    return ncz_def_var_extra(ncid, varid, NULL, NULL, NULL, NULL,
+			    NULL, NULL, NULL, NULL, NULL,
+			    &quantize_mode, &nsd);
+}
+
+/**
+Ensure that the quantize information for a variable is defined.
+Keep a flag in the NCZ_VAR_INFO_T struct to indicate if quantize
+info is defined, and if not, read the attribute.
+*/
+int
+NCZ_ensure_quantizer(int ncid, NC_VAR_INFO_T* var)
+{
+    int nsd = 0;
+
+    /* Read the attribute */
+    if(NCZ_get_att(ncid,var->hdr.id,NC_QUANTIZE_BITGROOM_ATT_NAME,&nsd,NC_INT)==NC_NOERR) {
+	var->quantize_mode = NC_QUANTIZE_BITGROOM;
+        var->nsd = nsd;
+    } else if(NCZ_get_att(ncid,var->hdr.id,NC_QUANTIZE_GRANULARBR_ATT_NAME,&nsd,NC_INT)==NC_NOERR) {
+	var->quantize_mode = NC_QUANTIZE_GRANULARBR;
+        var->nsd = nsd;
+    } else if(NCZ_get_att(ncid,var->hdr.id,NC_QUANTIZE_BITROUND_ATT_NAME,&nsd,NC_INT)==NC_NOERR) {
+	var->quantize_mode = NC_QUANTIZE_BITROUND;
+        var->nsd = nsd;
+    } else {
+	var->quantize_mode = NC_NOQUANTIZE;
+        var->nsd = 0;
+    }
+    if(var->quantize_mode < 0) var->quantize_mode = 0;
+    return NC_NOERR;
+}
+
+/**
+ * @internal Get quantize information about a variable. Pass NULL for
+ * whatever you don't care about. Note that this can require reading
+ * all the attributes for the variable.
+ *
+ * @param ncid File ID.
+ * @param varid Variable ID.
+ * @param quantize_modep Gets quantize mode.
+ * @param nsdp Gets Number of Significant Digits if quantize is in use.
+ *
+ * @returns ::NC_NOERR No error.
+ * @returns ::NC_EBADID Bad ncid.
+ * @returns ::NC_ENOTVAR Bad varid.
+ * @returns ::NC_EINVAL Invalid input.
+ * @author Ed Hartnett
+ */
+int
+NCZ_inq_var_quantize(int ncid, int varid, int *quantize_modep,
+		     int *nsdp)
+{
+    NC_VAR_INFO_T *var;
+    int retval;
+
+    LOG((2, "%s: ncid 0x%x varid %d", __func__, ncid, varid));
+
+    /* Find info for this file and group, and set pointer to each. */
+    /* Get pointer to the var. */
+    if ((retval = nc4_find_grp_h5_var(ncid, varid, NULL, NULL, &var)))
+        return retval;
+    if (!var)
+        return NC_ENOTVAR;	
+    assert(var->hdr.id == varid);
+    if(var->quantize_mode == -1)
+        {if((retval = NCZ_ensure_quantizer(ncid, var))) return retval;}
+    /* Copy the data to the user's data buffers. */
+    if (quantize_modep)
+        *quantize_modep = var->quantize_mode;
+    if (nsdp)
+        *nsdp = var->nsd;
+    return NC_NOERR;
+ }
+
 /**
  * @internal Rename a var to "bubba," for example. This is called by
  * nc_rename_var() for netCDF-4 files. This results in complexities
@@ -1313,6 +1553,7 @@ NCZ_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     size64_t stride[NC_MAX_VAR_DIMS];
     int retval, range_error = 0, i, d2;
     void *bufr = NULL;
+    int bufrd = 0; /* 1 => we allocated bufr */
     int need_to_convert = 0;
     int zero_count = 0; /* true if a count is zero */
     size_t len = 1;
@@ -1426,18 +1667,23 @@ NCZ_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     }
 #endif
 
-    /* Are we going to convert any data? (No converting of user-defined types
-       except enum). */
-    if (mem_nc_type != var->type_info->hdr.id &&
+    /* Are we going to convert any data? (No converting of compound or
+     * opaque or vlen types.) We also need to call this code if we are doing
+     * quantization. */
+    if ((mem_nc_type != var->type_info->hdr.id &&
 	mem_nc_type != NC_COMPOUND && mem_nc_type != NC_OPAQUE
 	&& mem_nc_type != NC_VLEN)
+	|| var->quantize_mode > 0)
     {
 	size_t file_type_size;
 
 	/* We must convert - allocate a buffer. */
 	need_to_convert++;
-        for (d2=0; d2<(var->ndims+zvar->scalar); d2++)
+	if(zvar->scalar)
+	    len = 1;
+        else for (d2=0; d2<var->ndims; d2++)
 	    len *= countp[d2];
+
 	LOG((4, "converting data for var %s type=%d len=%d", var->hdr.name,
 	     var->type_info->hdr.id, len));
 
@@ -1449,9 +1695,12 @@ NCZ_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
 	/* If we're reading, we need bufr to have enough memory to store
 	 * the data in the file. If we're writing, we need bufr to be
 	 * big enough to hold all the data in the file's type. */
-	if (len > 0)
+	if (len > 0) {
+	    assert(bufr == NULL);
 	    if (!(bufr = malloc(len * file_type_size)))
 		BAIL(NC_ENOMEM);
+	    bufrd = 1;
+	}
     }
     else
 	bufr = (void *)data;
@@ -1525,9 +1774,12 @@ NCZ_put_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     /* Do we need to convert the data? */
     if (need_to_convert)
     {
+	if(var->quantize_mode < 0) {if((retval = NCZ_ensure_quantizer(ncid,var))) BAIL(retval);}
+	assert(bufr != NULL);
 	if ((retval = nc4_convert_type(data, bufr, mem_nc_type, var->type_info->hdr.id,
 				       len, &range_error, var->fill_value,
-				       (h5->cmode & NC_CLASSIC_MODEL))))
+				       (h5->cmode & NC_CLASSIC_MODEL),
+				       var->quantize_mode, var->nsd)))
 	    BAIL(retval);
     }
 
@@ -1566,7 +1818,7 @@ exit:
     if (xfer_plistid && (H5Pclose(xfer_plistid) < 0))
 	BAIL2(NC_EPARINIT);
 #endif
-    if (need_to_convert && bufr) free(bufr);
+    if (bufrd && bufr) free(bufr);
 
     /* If there was an error return it, otherwise return any potential
        range error value. If none, return NC_NOERR as usual.*/
@@ -1620,7 +1872,6 @@ NCZ_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     size64_t fmaxdims[NC_MAX_VAR_DIMS];
     size64_t start[NC_MAX_VAR_DIMS];
     size64_t stride[NC_MAX_VAR_DIMS];
-    void *fillvalue = NULL;
     int no_read = 0, provide_fill = 0;
     int fill_value_size[NC_MAX_VAR_DIMS];
     int retval, range_error = 0, i, d2;
@@ -1704,8 +1955,12 @@ NCZ_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     {
         /* We must convert - allocate a buffer. */
         need_to_convert++;
-        for (d2 = 0; d2 < (var->ndims+zvar->scalar); d2++)
-            len *= countp[d2];
+	if(zvar->scalar) {
+	    len *= countp[0];	
+        } else {
+	    for (d2 = 0; d2 < (var->ndims); d2++)
+                len *= countp[d2];
+        }
         LOG((4, "converting data for var %s type=%d len=%d", var->hdr.name,
  		       var->type_info->hdr.id, len));
 
@@ -1860,8 +2115,8 @@ NCZ_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
 
 	/* Get the fill value from the ZARR variable. Memory will be
 	 * allocated. */
-	if (ncz_get_fill_value(h5, var, &fillvalue) < 0)
-	    BAIL(NC_EHDFERR);
+	if (NCZ_ensure_fill_value(var))
+	    BAIL(NC_EINVAL);
 
 	/* How many fill values do we need? */
 	for (fill_len = 1, d2 = 0; d2 < var->ndims; d2++)
@@ -1871,28 +2126,9 @@ NCZ_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
 	filldata = (char *)data + real_data_size;
 	for (i = 0; i < fill_len; i++)
 	{
-
-	    if (var->type_info->nc_type_class == NC_STRING)
-	    {
-		if (*(char **)fillvalue)
-		{
-		    if (!(*(char **)filldata = strdup(*(char **)fillvalue)))
-			BAIL(NC_ENOMEM);
-		}
-		else
-		    *(char **)filldata = NULL;
-	    }
-	    else if (var->type_info->nc_type_class == NC_VLEN)
-	    {
-		if (fillvalue)
-		{
-		    memcpy(filldata,fillvalue,file_type_size);
-		} else {
-		    *(char **)filldata = NULL;
-		}
-	    }
-	    else
-		memcpy(filldata, fillvalue, file_type_size);
+	    /* Copy one instance of the fill_value */
+	    if((retval = nc_copy_data(ncid,var->type_info->hdr.id,var->fill_value,1,filldata)))
+	        BAIL(retval);
 	    filldata = (char *)filldata + file_type_size;
 	}
     }
@@ -1900,9 +2136,11 @@ NCZ_get_vars(int ncid, int varid, const size_t *startp, const size_t *countp,
     /* Convert data type if needed. */
     if (need_to_convert)
     {
+	if(var->quantize_mode < 0) {if((retval = NCZ_ensure_quantizer(ncid,var))) BAIL(retval);}
 	if ((retval = nc4_convert_type(bufr, data, var->type_info->hdr.id, mem_nc_type,
 					   len, &range_error, var->fill_value,
-					   (h5->cmode & NC_CLASSIC_MODEL))))
+				           (h5->cmode & NC_CLASSIC_MODEL), var->quantize_mode,
+				           var->nsd)))
 	   BAIL(retval);
         /* For strict netcdf-3 rules, ignore erange errors between UBYTE
 	 * and BYTE types. */
@@ -1928,15 +2166,6 @@ exit:
 #endif
     if (need_to_convert && bufr)
 	free(bufr);
-    if (fillvalue)
-    {
-	if (var->type_info->nc_type_class == NC_VLEN)
-	    nc_free_vlen((nc_vlen_t *)fillvalue);
-	else if (var->type_info->nc_type_class == NC_STRING && *(char **)fillvalue)
-	    free(*(char **)fillvalue);
-	free(fillvalue);
-    }
-
     /* If there was an error return it, otherwise return any potential
        range error value. If none, return NC_NOERR as usual.*/
     if (retval)
@@ -2002,10 +2231,22 @@ NCZ_inq_var_all(int ncid, int varid, char *name, nc_type *xtypep,
 	goto done;
     assert(grp && h5);
 
+    /* Short-circuit the filter-related inquiries */
+    if(shufflep) {
+	*shufflep = 0;
+	if((retval = NCZ_inq_var_filter_info(ncid,varid,2,NULL,NULL))==NC_NOERR)
+	    *shufflep = 1;
+    }
+    if(fletcher32p) {
+	*fletcher32p = 0;
+	if((retval = NCZ_inq_var_filter_info(ncid,varid,3,NULL,NULL))==NC_NOERR)
+	    *fletcher32p = 1;
+    }
+
     /* Now that lazy atts have been read, use the libsrc4 function to
      * get the answers. */
     retval = NC4_inq_var_all(ncid, varid, name, xtypep, ndimsp, dimidsp, nattsp,
-			   shufflep, unused4, unused5, fletcher32p,
+			   NULL, unused4, unused5, NULL,
 			   storagep, chunksizesp, no_fill, fill_valuep,
 			   endiannessp, unused1, unused2, unused3);
 done:
